#include "bb_sudoku_solver.h"
#include "bb_sudoku_tables.h"

static bool IsValidCompletion( char const* puzzle, unsigned const* completion )
   {
   // first make sure that the completion actually is a correctly completed Sudoku
   for ( int g = 0;  g < 27;  g++ )
      {  
      unsigned digits = 0;
      for ( int i = 0;  i < 9;  i++ )
         digits |= completion[ Group[ g ][ i ] ];
      if ( digits != 0x1FF )
         return false;
      }
   // now check that the completion is a completion of the given puzzle
   for ( int i = 0;  i < 81;  i++ )
      if ( puzzle[ i ] != '0'  &&  puzzle[ i ] != (B2V[ completion[ i ] ] + '0') )
         return false;
   return true;
   }

int GetNumberOfCompletionsOfPuzzle( char const* puzzle )
   {
   BritSolver solver;
   unsigned completions[ 2 ][ 81 ];
   int const numcomp = solver.GetNumberOfCompletions( puzzle, completions );
   // if two completions were found make sure that
   //   (i)  the completions are in fact valid completions of the given puzzle;
   //   (ii) the completions are different.
   if ( numcomp > 1  &&  (! IsValidCompletion( puzzle, completions[ 0 ] )  ||
        ! IsValidCompletion( puzzle, completions[ 1 ] )  ||  0 == memcmp( completions[ 0 ], completions[ 1 ], 81 * 4 )) )
      return -1;
   return numcomp;
   }
